import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'square_ui.dart';
import 'resizable_draggable.dart';
import 'controller.dart';
import 'constants.dart';

/// Chessboard Widget, receives size and a chessboard controller
class ChessBoard extends StatefulWidget {
  ChessBoard({
    required this.initialSize,
    required this.controller,
    this.whiteSideTowardsUser = true,
    this.minSize = 400,
    this.boardType = BoardType.brown,
    this.notationPanelBackgroundColor = Colors.black,
    this.notationPanelWidth = 200,
    this.notationColor = Colors.white,
    this.showNotationPanel = true,
    this.containerColor = Colors.transparent,
    this.containerRadius = const Radius.circular(10),
    this.notationTopBar = const Padding(
      padding: const EdgeInsets.all(8.0),
      child: Text('Notation', style: TextStyle(color: Colors.white)),
    ),
  })  : assert(
          initialSize >= minSize,
          'size property must be greater or equal to minSize',
        ),
        assert(notationPanelWidth >= 200,
            'notationPanelWidth must be at least 200');

  final ChessboardController controller;
  final double minSize;
  final double initialSize;
  final BoardType boardType;
  final bool whiteSideTowardsUser;
  final Color notationPanelBackgroundColor;
  final double notationPanelWidth;
  final Color notationColor;
  final bool showNotationPanel;
  final Color containerColor;
  final Radius containerRadius;
  final Widget? notationTopBar;

  @override
  _ChessBoardState createState() => _ChessBoardState();
}

class _ChessBoardState extends State<ChessBoard> {
  late double size;
  late bool whiteTowardUser;
  @override
  void initState() {
    size = widget.initialSize;
    whiteTowardUser = widget.whiteSideTowardsUser;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    /// Make sure the board size wont get messed up
    final screenSize = MediaQuery.of(context).size;
    final smallestSide = screenSize.width < screenSize.height
        ? screenSize.width
        : screenSize.height;
    if (size > smallestSide) {
      size = smallestSide;
    }
    if (widget.initialSize < widget.minSize) size = widget.minSize;

    return Container(
      decoration: BoxDecoration(
          borderRadius: BorderRadius.all(widget.containerRadius),
          color: widget.containerColor),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Container(
            width: size,
            height: size,
            child: Stack(
              children: <Widget>[
                Center(
                  child: Container(
                    width: size * 0.95,
                    height: size * 0.95,
                    child: getBoardImage(widget.boardType),
                  ),
                ),

                /// the pieces on the chessboard
                /// meaning, state of each square
                Center(
                  child: FittedBox(
                    fit: BoxFit.contain,
                    child: Container(
                      width: size * 0.95,
                      height: size * 0.95,
                      child: buildChessBoard(),
                    ),
                  ),
                ),
                Positioned(
                    right: 0,
                    bottom: 0,
                    child: ResizableDraggable(
                      size: size / 20,
                      onDrag: (double offset) {
                        if (size + offset >= widget.minSize &&
                            size + offset <= smallestSide)
                          setState(() {
                            size = widget.initialSize + offset;
                          });
                      },
                    )),
              ],
            ),
          ),
          if ((screenSize.width > size + 20 + widget.notationPanelWidth) &&
              screenSize.height > 300 &&
              widget.showNotationPanel)
            Padding(
              padding: const EdgeInsets.only(right: 8.0),
//              child: ChessboardNotation(
//                flipBoard: flipBoard,
//                width: widget.notationPanelWidth,
//                height: size * 0.95,
//                backgroundColor: widget.notationPanelBackgroundColor,
//                controller: widget.controller,
//                notationTextColor: widget.notationColor,
//                notationTopBar: widget.notationTopBar,
////                onTapMove: (int) {
////                  setState(() {
////                    /// TODO create a function to set position from history state
////                  });
////                },
//              ),
            ),
        ],
      ),
    );
  }

  void flipBoard() {
    setState(() {
      whiteTowardUser = !whiteTowardUser;
    });
  }

  /// Builds the board, drawing each square
  Widget buildChessBoard() {
    final pieceSize = size / 8;
    return Column(
      children: whiteTowardUser
          ? squareList.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row
                      .map((squareName) => BoardSquare(
                            key: ValueKey(squareName),
                            squareName: squareName,
                            size: pieceSize,
                            controller: widget.controller,
                            onMove: (move) {
                              widget.controller.makePrettyMove(move);
                            },
                          ))
                      .toList(),
                ),
              );
            }).toList()
          : squareList.reversed.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row.reversed
                      .map((squareName) => BoardSquare(
                            key: ValueKey(squareName),
                            squareName: squareName,
                            size: widget.initialSize,
                            controller: widget.controller,
                            onMove: (move) {
                              widget.controller.makePrettyMove(move);
                            },
                          ))
                      .toList(),
                ),
              );
            }).toList(),
    );
  }

  /// Returns the board image
  Image? getBoardImage(BoardType boardType) {
    switch (boardType) {
      case BoardType.brown:
        return Image.asset(
          'packages/chessboard_js/images/brown_board.png',
          fit: BoxFit.contain,
        );
      case BoardType.darkBrown:
        return Image.asset(
          "packages/chessboard_js/images/dark_brown_board.png",
          fit: BoxFit.cover,
        );
      case BoardType.green:
        return Image.asset(
          "packages/chessboard_js/images/green_board.png",
          fit: BoxFit.cover,
        );
      case BoardType.orange:
        return Image.asset(
          "packages/chessboard_js/images/orange_board.png",
          fit: BoxFit.cover,
        );
      default:
        return null;
    }
  }
}
