import 'package:chessjs/chessjs.dart' as chessjs;
import 'package:chessjs/pieces.dart';
import 'chessboard_model.dart';

class ChessboardController {
  ChessboardController({bool movesEnabled = true}) {
    this._enableMoves = movesEnabled;
    buildBoard();
  }

  /// Todo be able to pass a logic, for example a game that already started
  /// and setup the board from a determined position
//  ChessboardController(chessjs.Chess? logic){
//
//    if(logic == null){
//      logic = chessjs.Chess();
//    }
//
//    this.logic = logic;
//
//  }

  final ChessboardModel _chessboard = ChessboardModel();
  final chessjs.Chess _logic = chessjs.Chess();
  late bool _enableMoves;

  ChessboardModel get board => _chessboard;
  chessjs.Chess get logic => _logic;
  bool get movesEnabled => _enableMoves;

  buildBoard() {
    _chessboard.squares.forEach((square) {
      Piece? piece = _logic.get(square.squareName);
      if (piece != null) {
        square.squareState = piece;
      }
    });
  }

  List<String> getSanHistory() {
    List<String> history = _logic.getHistorySAN();
    return history;
  }

  /// Get history of game (moves) with details for each move.
  /// Returns a List<Map>
  List<Map> getVerboseHistory() {
    List<Map> history = _logic.getHistoryVerbose();
    return history;
  }

  void toggleEnableMove() {
    _enableMoves = !_enableMoves;
  }

  /// update state of a specific square provided
  /// its squareName and the state from the logic.
  updateSquare(String squareName) {
//    print('update: $squareName with ${_logic.get(squareName)}');
    _chessboard
        //.squares.firstWhere((square) => squareName == square.squareName)
        .updateSquareState(
      squareName,
      _logic.get(squareName),
    );
  }

  /// returns true if successfully make the move
  /// otherwise false
  bool makeSanMove(String move) {
    chessjs.Move? moveObj;
    final moves = _logic.generateMoves();
    for (int i = 0; i < moves.length; i++) {
      if (move == _logic.moveToSan(moves[i])) {
        moveObj = moves[i];
        break;
      }
    }
    if (moveObj != null) {
      _logic.makeMove(moveObj);
      _updateBoardAfterMove(moveObj);
      return true;
    } else
      return false;
  }

  /// Attempt to make a move in the board if successfully made the move,
  /// return true and update the respective squares, otherwise return false.
  bool makePrettyMove(Map<String, String> move) {
    final builtMove = buildPrettyMove(move);
    if (builtMove != null) {
      _logic.makeMove(builtMove);
      _updateBoardAfterMove(builtMove);
      return true;
    } else
      return false;
  }

  chessjs.Move? buildPrettyMove(Map<String, String> move) {
    final String? promotionStr = move['promotion'];
    final PieceType? promotionPiece;
    switch (promotionStr) {
      case 'n':
        promotionPiece = PieceType.KNIGHT;
        break;
      case 'b':
        promotionPiece = PieceType.BISHOP;
        break;
      case 'r':
        promotionPiece = PieceType.ROOK;
        break;
      case 'q':
        promotionPiece = PieceType.QUEEN;
        break;
      default:
        promotionPiece = null;
    }

    final possibleMoves = _logic.generateMoves();
    chessjs.Move? builtMove;
    possibleMoves.forEach((possibleMove) {
      if (possibleMove.fromAlgebraic == move['from'] &&
          possibleMove.toAlgebraic == move['to'] &&
          possibleMove.promotion == promotionPiece) {
        builtMove = possibleMove;
      }
    });

    if (builtMove == null) {
      return null;
    } else
      return builtMove;
  }

  _updateBoardAfterMove(chessjs.Move move) {
    int flags = move.flags;

    /// update square from where the move originated
    updateSquare(move.fromAlgebraic);

    /// if castling, update squares associated
    if ((flags & chessjs.BITS_KSIDE_CASTLE) != 0) {
      if (move.color == chessjs.WHITE) {
        updateSquare('h1');
        updateSquare('g1');
        updateSquare('f1');
      } else {
        updateSquare('h8');
        updateSquare('g8');
        updateSquare('f8');
      }
    } else if ((flags & chessjs.BITS_QSIDE_CASTLE) != 0) {
      if (move.color == chessjs.WHITE) {
        updateSquare('a1');
        updateSquare('c1');
        updateSquare('d1');
      } else {
        updateSquare('a8');
        updateSquare('c8');
        updateSquare('d8');
      }
    }

    /// if en passant, update squares associated
    else if ((flags & (chessjs.BITS_EP_CAPTURE)) != 0) {
      late final squareToUpdate;

      String to = move.toAlgebraic;
      if (move.color == chessjs.WHITE) {
        squareToUpdate = to.replaceAll('6', '5');
      } else if (move.color == chessjs.BLACK) {
        squareToUpdate = to.replaceAll('3', '4');
      }
      updateSquare(squareToUpdate);
      updateSquare(move.fromAlgebraic);
      updateSquare(move.toAlgebraic);
    }

    /// otherwise just update the square where the move finished
    else {
      updateSquare(move.toAlgebraic);
    }
  }
}
